# frozen_string_literal: true
require 'horn_of_plenty/collection'
require 'horn_of_plenty/adapters/github/repositories/issue'
require 'horn_of_plenty/adapters/github/repositories/pull_request'

module  HornOfPlenty
module  Adapters
module  Github
module  Collections
class   Card < HornOfPlenty::Collection
  protected

  # rubocop:disable Metrics/AbcSize
  def transform(raw_item)
    card = super

    if %w{Issue PullRequest}.include? card.content_type
      issues = Repositories::Issue.fetch(
                 repository: card.repository,
                 query:      { id: card.content_id },
      )

      card.content_type = issues.first.has_pull_request? ? 'PullRequest' : 'Issue'
      card.title        = issues.first.title
      card.description  = issues.first.description
      card.labels       = issues.first.labels
      card.status       = issues.first.status

      if issues.first.has_pull_request?
        pull_requests = Repositories::PullRequest.fetch(
                          repository: card.repository,
                          query:      { id: card.content_id },
        )

        card.status = pull_requests.first.status if pull_requests.first
      end
    end

    card
  end
  # rubocop:enable Metrics/AbcSize
end
end
end
end
end
