# frozen_string_literal: true
require 'horn_of_plenty/core_ext/string'
require 'faraday'

module  HornOfPlenty
module  Adapters
module  Github
class   Error
  # rubocop:disable Metrics/LineLength
  STATUS_CODE_ERROR_MAPPING = {
    400 => ['BadRequest',            'There was a general issue with your request.'],
    401 => ['AuthenticationFailure', 'There was a problem with the credentials you provided.'],
    403 => ['Forbidden',             'You are not authorized to access this resource.'],
    404 => ['NotFound',              'The resource you requested cannot be found.'],
    422 => ['UnprocessableEntity',   'We could not process your request.'],
    500 => ['ServerError',           'Something went wrong on the server.'],
  }.freeze
  # rubocop:enable Metrics/LineLength

  def self.lookup(http_status_code, message = nil)
    error_data = STATUS_CODE_ERROR_MAPPING[http_status_code]

    fail StandardError, "Unknown Status Code: #{http_status_code}" unless error_data

    error_namespace  = 'HornOfPlenty::Adapters::Github::Errors'
    error_class      = HornOfPlenty::CoreExt::String.constantize(
                         "#{error_namespace}::#{error_data[0]}",
    )

    error_class.new("#{error_data[1]}\n#{message}")
  end
end

module Errors
  Github::Error::STATUS_CODE_ERROR_MAPPING.each do |_code, (error_name, _error_message)|
    module_eval <<-HEREDOC
      class #{error_name} < StandardError
      end
    HEREDOC
  end
end
end
end
end
