# frozen_string_literal: true
require 'horn_of_plenty/parsers/hash'
require 'horn_of_plenty/null_objects/board'
require 'horn_of_plenty/models/board'

module  HornOfPlenty
module  Adapters
module  Github
module  Parsers
class   Board
  include HornOfPlenty::Parsers::Hash

  attr_accessor :raw

  def initialize(raw:)
    self.raw = raw
  end

  def id
    @id                                    ||= parse_text(raw, 'number')
  end

  def author_id
    @author_id                             ||= parse_text(raw, 'creator/:/login')
  end

  def title
    @title                                 ||= parse_text(raw, 'name')
  end

  def description
    @description                           ||= parse_text(raw, 'body')
  end

  def created_at
    @created_at                            ||= parse_time(raw, 'created_at')
  end

  def updated_at
    @updated_at                            ||= parse_time(raw, 'updated_at')
  end

  def self.to_model(raw_item)
    return NullObjects::Board.instance unless raw_item

    Models::Board.from_parser(parser: new(raw: raw_item))
  end
end
end
end
end
end
