# frozen_string_literal: true
require 'horn_of_plenty/parsers/hash'
require 'horn_of_plenty/null_objects/card'
require 'horn_of_plenty/models/card'
require 'horn_of_plenty/core_ext/string'

# rubocop:disable Style/SpaceAroundOperators
module  HornOfPlenty
module  Adapters
module  Github
module  Parsers
class   Card
  include HornOfPlenty::Parsers::Hash

  attr_accessor :raw

  def initialize(raw:)
    self.raw = raw
  end

  def id
    @id                                    ||= parse_text(raw, 'id')
  end

  def lane_id
    @lane_id                               ||= parse_text(raw, 'column_url') do |url|
      url.match(%r{\A.*/(\d+)\z})[1]
    end
  end

  def repository
    @repository                            ||= parse_text(raw, 'column_url') do |url|
      repo_name = url.match(%r{\A.*/repos/((?:[^/]+/){2}).*\z})[1]

      repo_name && repo_name[0..-2]
    end
  end

  def content_type
    @content_type                          ||= parse_text(raw,
                                                          'content_url',
                                                          yield_if_nil: true) do |url|
      if url
        CoreExt::String.singularize(
          CoreExt::String.camelize(url.match(%r{\A.*/(.*?)/\d+\z})[1]),
        )
      else
        'Note'
      end
    end
  end

  def content_id
    @content_id                            ||= parse_text(raw, 'content_url') do |url|
      url.match(%r{\A.*/(\d+)\z})[1]
    end
  end

  def title
    @title                                 ||= parse_text(raw, 'note') do |note|
      note.delete("\r").match(/\A(.*)$/)[1]
    end
  end

  def description
    @description                           ||= parse_text(raw, 'note') do |note|
      match = note.delete("\r").match(/\A.*?(?:\n)+(.*)/m)

      match[1] if match
    end
  end

  def created_at
    @created_at                            ||= parse_time(raw, 'created_at')
  end

  def updated_at
    @updated_at                            ||= parse_time(raw, 'updated_at')
  end

  def self.to_model(raw_item)
    return NullObjects::Card.instance unless raw_item

    Models::Card.from_parser(parser: new(raw: raw_item))
  end
end
end
end
end
end
# rubocop:enable Style/SpaceAroundOperators
