# frozen_string_literal: true
require 'horn_of_plenty/parsers/hash'
require 'horn_of_plenty/null_objects/issue'
require 'horn_of_plenty/models/issue'

# rubocop:disable Style/SpaceAroundOperators
module  HornOfPlenty
module  Adapters
module  Github
module  Parsers
class   Issue
  include HornOfPlenty::Parsers::Hash

  attr_accessor :raw

  def initialize(raw:)
    self.raw = raw
  end

  def id
    @id                                    ||= parse_text(raw, 'number')
  end

  def author_id
    @author_id                             ||= parse_text(raw, 'user/:/login')
  end

  def assignee_ids
    @assignee_ids                          ||= begin
      parse_key(raw, 'assignees').map { |assignee| assignee['login'] }
    end
  end

  def closer_id
    @closer_id                             ||= parse_text(raw, 'closed_by/:/login')
  end

  def repository
    @repository                            ||= parse_text(raw, 'url') do |url|
      repo_name = url.match(%r{\A.*/repos/((?:[^/]+/){2}).*\z})[1]

      repo_name && repo_name[0..-2]
    end
  end

  def status
    @status                                ||= parse_text(raw, 'state')
  end

  def title
    @title                                 ||= parse_text(raw, 'title')
  end

  def description
    @description                           ||= parse_text(raw, 'body') do |description|
      description.delete("\r")
    end
  end

  def labels
    @labels                                ||= begin
      labels    = parse_key(raw, 'labels').map { |label| label['name'] }
      milestone = Array(parse_key(raw, 'milestone/:/title'))

      labels + milestone
    end
  end

  def closed_at
    @closed_at                             ||= parse_time(raw, 'closed_at')
  end

  def created_at
    @created_at                            ||= parse_time(raw, 'created_at')
  end

  def updated_at
    @updated_at                            ||= parse_time(raw, 'updated_at')
  end

  def has_pull_request?
    @has_pull_request                      ||= parse_existance(raw, 'pull_request')
  end

  def self.to_model(raw_item)
    return NullObjects::Issue.instance unless raw_item

    Models::Issue.from_parser(parser: new(raw: raw_item))
  end
end
end
end
end
end
# rubocop:enable Style/SpaceAroundOperators
