# frozen_string_literal: true
require 'forwardable'

module  HornOfPlenty
class   Collection
  include Enumerable
  extend  Forwardable

  def_delegators :raw_items,
                 :length,
                 :size,
                 :count,
                 :empty?

  def initialize(items:, parser:)
    self.raw_items = items
    self.parser    = parser
  end

  def [](index)
    items[index] ||= transform(raw_items[index])
  end

  def first
    items[0] ||= transform(raw_items[0])
  end

  def shift(*args)
    raw_items.shift(*args)
    items.shift(*args)
  end

  def pop(*args)
    raw_items.pop(*args)
    items.pop(*args)
  end

  def each
    raw_items.each_with_index do |raw_item, index|
      items[index] = items[index] || transform(raw_item)

      yield items[index]
    end
  end

  protected

  attr_accessor :raw_items,
                :items,
                :parser

  def transform(raw_item)
    parser.to_model(raw_item)
  end

  def items
    @items ||= []
  end
end
end
