# frozen_string_literal: true
require 'json'
require 'horn_of_plenty/collection'

module  HornOfPlenty
class   Response
  attr_accessor :raw_response

  def initialize(raw_response:)
    self.raw_response = raw_response
  end

  def result
    @result ||= ::HornOfPlenty::Collection.new(items:  items,
                                               parser: parser_class)
  end

  def items
    @items ||= body.is_a?(Array) ? body : [body]
  end

  def self.parse(raw_response)
    new(raw_response: raw_response).tap do |response|
      fail response.error unless response.successful?
    end
  end

  def body
    @body ||= JSON.parse(raw_response.body) || ''
  end

  def status_code
    @status_code ||= raw_response.status
  end

  def status_message
    @status_message ||= body['message']
  end

  def successful?
    (200..299).cover? status_code
  end
end
end
